@@ -1,7 +1,7 @@
 # frozen_string_literal: true
 
 # Redmine - project management software
-# Copyright (C) 2006-2021  Jean-Philippe Lang
+# Copyright (C) 2006-  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -17,7 +17,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
-class Project < ActiveRecord::Base
+class Project < ApplicationRecord
   include Redmine::SafeAttributes
   include Redmine::NestedSet::ProjectNestedSet
 
@@ -25,16 +25,16 @@ class Project < ActiveRecord::Base
   STATUS_ACTIVE     = 1
   STATUS_CLOSED     = 5
   STATUS_ARCHIVED   = 9
+  STATUS_SCHEDULED_FOR_DELETION = 10
 
   # Maximum length for project identifiers
   IDENTIFIER_MAX_LENGTH = 100
 
-  # Specific overridden Activities
-  has_many :time_entry_activities
   has_many :memberships, :class_name => 'Member', :inverse_of => :project
   # Memberships of active users only
   has_many :members,
            lambda {joins(:principal).where(:users => {:type => 'User', :status => Principal::STATUS_ACTIVE})}
+  has_many :users, through: :members
   has_many :enabled_modules, :dependent => :delete_all
   has_and_belongs_to_many :trackers, lambda {order(:position)}
   has_many :issues, :dependent => :destroy
@@ -43,7 +43,9 @@ class Project < ActiveRecord::Base
   belongs_to :default_version, :class_name => 'Version'
   belongs_to :default_assigned_to, :class_name => 'Principal'
   has_many :time_entries, :dependent => :destroy
-  has_many :queries, :dependent => :delete_all
+  # Specific overridden Activities
+  has_many :time_entry_activities, :dependent => :destroy
+  has_many :queries, :dependent => :destroy
   has_many :documents, :dependent => :destroy
   has_many :news, lambda {includes(:author)}, :dependent => :destroy
   has_many :issue_categories, lambda {order(:name)}, :dependent => :delete_all
@@ -58,6 +60,8 @@ class Project < ActiveRecord::Base
                           :class_name => 'IssueCustomField',
                           :join_table => "#{table_name_prefix}custom_fields_projects#{table_name_suffix}",
                           :association_foreign_key => 'custom_field_id'
+  # Default Custom Query
+  belongs_to :default_issue_query, :class_name => 'IssueQuery'
 
   acts_as_attachable :view_permission => :view_files,
                      :edit_permission => :manage_files,
@@ -72,7 +76,7 @@ class Project < ActiveRecord::Base
                 :author => nil
 
   validates_presence_of :name, :identifier
-  validates_uniqueness_of :identifier, :if => proc {|p| p.identifier_changed?}
+  validates_uniqueness_of :identifier, :if => proc {|p| p.identifier_changed?}, :case_sensitive => true
   validates_length_of :name, :maximum => 255
   validates_length_of :homepage, :maximum => 255
   validates_length_of :identifier, :maximum => IDENTIFIER_MAX_LENGTH
@@ -83,13 +87,13 @@ class Project < ActiveRecord::Base
   validates_exclusion_of :identifier, :in => %w(new)
   validate :validate_parent
 
+  after_update :update_versions_from_hierarchy_change,
+               :if => proc {|project| project.saved_change_to_parent_id?}
+  before_destroy :delete_all_members
   after_save :update_inherited_members,
              :if => proc {|project| project.saved_change_to_inherit_members?}
   after_save :remove_inherited_member_roles, :add_inherited_member_roles,
              :if => proc {|project| project.saved_change_to_parent_id?}
-  after_update :update_versions_from_hierarchy_change,
-               :if => proc {|project| project.saved_change_to_parent_id?}
-  before_destroy :delete_all_members
 
   scope :has_module, (lambda do |mod|
     where("#{Project.table_name}.id IN (SELECT em.project_id FROM #{EnabledModule.table_name} em WHERE em.name=?)", mod.to_s)
@@ -105,8 +109,8 @@ class Project < ActiveRecord::Base
   end)
   scope :like, (lambda do |arg|
     if arg.present?
-      pattern = "%#{arg.to_s.strip}%"
-      where("LOWER(identifier) LIKE LOWER(:p) OR LOWER(name) LIKE LOWER(:p)", :p => pattern)
+      pattern = "%#{sanitize_sql_like arg.to_s.strip}%"
+      where("LOWER(#{Project.table_name}.identifier) LIKE LOWER(:p) ESCAPE :s OR LOWER(#{Project.table_name}.name) LIKE LOWER(:p) ESCAPE :s", :p => pattern, :s => '\\')
     end
   end)
   scope :sorted, lambda {order(:lft)}
@@ -117,6 +121,7 @@ class Project < ActiveRecord::Base
   def initialize(attributes=nil, *args)
     super
 
+    # rubocop:disable Style/NegatedIf
     initialized = (attributes || {}).stringify_keys
     if !initialized.key?('identifier') && Setting.sequential_project_identifiers?
       self.identifier = Project.next_identifier
@@ -135,6 +140,7 @@ class Project < ActiveRecord::Base
         self.trackers = Tracker.sorted.to_a
       end
     end
+    # rubocop:enable Style/NegatedIf
   end
 
   def identifier=(identifier)
@@ -180,7 +186,7 @@ class Project < ActiveRecord::Base
     perm = Redmine::AccessControl.permission(permission)
     base_statement =
       if perm && perm.read?
-        "#{Project.table_name}.status <> #{Project::STATUS_ARCHIVED}"
+        "#{Project.table_name}.status IN (#{Project::STATUS_ACTIVE}, #{Project::STATUS_CLOSED})"
       else
         "#{Project.table_name}.status = #{Project::STATUS_ACTIVE}"
       end
@@ -326,15 +332,17 @@ class Project < ActiveRecord::Base
   # Returns a :conditions SQL string that can be used to find the issues associated with this project.
   #
   # Examples:
-  #   project.project_condition(true)  => "(projects.id = 1 OR (projects.lft > 1 AND projects.rgt < 10))"
+  #   project.project_condition(true)  => "(projects.lft >= 1 AND projects.rgt <= 10)"
   #   project.project_condition(false) => "projects.id = 1"
   def project_condition(with_subprojects)
-    cond = "#{Project.table_name}.id = #{id}"
     if with_subprojects
-      cond = "(#{cond} OR (#{Project.table_name}.lft > #{lft} AND " \
-               "#{Project.table_name}.rgt < #{rgt}))"
+      "(" \
+        "#{Project.table_name}.lft >= #{lft} AND " \
+        "#{Project.table_name}.rgt <= #{rgt}" \
+      ")"
+    else
+      "#{Project.table_name}.id = #{id}"
     end
-    cond
   end
 
   def self.find(*args)
@@ -350,12 +358,12 @@ class Project < ActiveRecord::Base
     end
   end
 
-  def self.find_by_param(*args)
-    self.find(*args)
+  def self.find_by_param(*)
+    self.find(*)
   end
 
   alias :base_reload :reload
-  def reload(*args)
+  def reload(*)
     @principals = nil
     @users = nil
     @shared_versions = nil
@@ -373,7 +381,8 @@ class Project < ActiveRecord::Base
     @due_date = nil
     @override_members = nil
     @assignable_users = nil
-    base_reload(*args)
+    @last_activity_date = nil
+    base_reload(*)
   end
 
   def to_param
@@ -397,6 +406,10 @@ class Project < ActiveRecord::Base
     self.status == STATUS_ARCHIVED
   end
 
+  def scheduled_for_deletion?
+    self.status == STATUS_SCHEDULED_FOR_DELETION
+  end
+
   # Archives the project and its descendants
   def archive
     # Check that there is no issue of a non descendant project that is assigned
@@ -483,6 +496,7 @@ class Project < ActiveRecord::Base
   def rolled_up_statuses
     issue_status_ids = WorkflowTransition.
       where(:tracker_id => rolled_up_trackers.map(&:id)).
+      where('old_status_id <> new_status_id').
       distinct.
       pluck(:old_status_id, :new_status_id).
       flatten.
@@ -557,14 +571,6 @@ class Project < ActiveRecord::Base
     end
   end
 
-  # TODO: Remove this method in Redmine 5.0
-  def members_by_role
-    ActiveSupport::Deprecation.warn(
-      "Project#members_by_role will be removed. Use Project#principals_by_role instead."
-    )
-    principals_by_role
-  end
-
   # Adds user as a project member with the default role
   # Used for when a non-admin user creates a project
   def add_default_member(user)
@@ -624,13 +630,7 @@ class Project < ActiveRecord::Base
 
   # Returns the users that should be notified on project events
   def notified_users
-    # TODO: User part should be extracted to User#notify_about?
-    users =
-      members.preload(:principal).select do |m|
-        m.principal.present? &&
-         (m.mail_notification? || m.principal.mail_notification == 'all')
-      end
-    users.collect {|m| m.principal}
+    users.where('members.mail_notification = ? OR users.mail_notification = ?', true, 'all')
   end
 
   # Returns a scope of all custom fields enabled for project issues
@@ -670,6 +670,8 @@ class Project < ActiveRecord::Base
   end
 
   def <=>(project)
+    return nil unless project.is_a?(Project)
+
     name.casecmp(project.name)
   end
 
@@ -824,7 +826,6 @@ class Project < ActiveRecord::Base
     'name',
     'description',
     'homepage',
-    'is_public',
     'identifier',
     'custom_field_values',
     'custom_fields',
@@ -832,8 +833,25 @@ class Project < ActiveRecord::Base
     'issue_custom_field_ids',
     'parent_id',
     'default_version_id',
+    'default_issue_query_id',
     'default_assigned_to_id')
 
+  safe_attributes(
+    'is_public',
+    :if =>
+      lambda do |project, user|
+        if project.new_record?
+          if user.admin?
+            true
+          else
+            default_member_role&.has_permission?(:select_project_publicity)
+          end
+        else
+          user.allowed_to?(:select_project_publicity, project)
+        end
+      end
+  )
+
   safe_attributes(
     'enabled_module_names',
     :if =>
@@ -842,7 +860,7 @@ class Project < ActiveRecord::Base
           if user.admin?
             true
           else
-            default_member_role.has_permission?(:select_project_modules)
+            default_member_role&.has_permission?(:select_project_modules)
           end
         else
           user.allowed_to?(:select_project_modules, project)
@@ -886,7 +904,7 @@ class Project < ActiveRecord::Base
       attrs['custom_fields'].reject! {|c| !editable_custom_field_ids.include?(c['id'].to_s)}
     end
 
-    super(attrs, user)
+    super
   end
 
   # Returns an auto-generated project identifier based on the last identifier used
@@ -911,7 +929,7 @@ class Project < ActiveRecord::Base
   #   project.copy(1, :only => 'members')                # => copies members only
   #   project.copy(1, :only => ['members', 'versions'])  # => copies members and versions
   def copy(project, options={})
-    project = project.is_a?(Project) ? project : Project.find(project)
+    project = Project.find(project) unless project.is_a?(Project)
 
     to_be_copied = %w(members wiki versions issue_categories issues queries boards documents)
     to_be_copied = to_be_copied & Array.wrap(options[:only]) unless options[:only].nil?
@@ -925,7 +943,7 @@ class Project < ActiveRecord::Base
         end
 
         to_be_copied.each do |name|
-          send "copy_#{name}", project
+          send :"copy_#{name}", project
         end
         Redmine::Hook.call_hook(:model_project_copy_before_save,
                                 :source_project => project,
@@ -939,7 +957,7 @@ class Project < ActiveRecord::Base
 
   # Returns a new unsaved Project instance with attributes copied from +project+
   def self.copy_from(project)
-    project = project.is_a?(Project) ? project : Project.find(project)
+    project = Project.find(project) unless project.is_a?(Project)
     # clear unique attributes
     attributes =
       project.attributes.dup.except('id', 'name', 'identifier',
@@ -968,6 +986,17 @@ class Project < ActiveRecord::Base
     end
   end
 
+  # Overrides Redmine::Acts::Customizable::InstanceMethods#validate_custom_field_values
+  # so that custom values that are not editable are not validated (eg. a custom field that
+  # is marked as required should not trigger a validation error if the user is not allowed
+  # to edit this field).
+  def validate_custom_field_values
+    user = User.current
+    if new_record? || custom_field_values_changed?
+      editable_custom_field_values(user).each(&:validate_value)
+    end
+  end
+
   # Returns the custom_field_values that can be edited by the given user
   def editable_custom_field_values(user=nil)
     visible_custom_field_values(user)
@@ -980,6 +1009,20 @@ class Project < ActiveRecord::Base
     end
   end
 
+  def last_activity_date
+    @last_activity_date || fetch_last_activity_date
+  end
+
+  # Preloads last activity date for a collection of projects
+  def self.load_last_activity_date(projects, user=User.current)
+    if projects.any?
+      last_activities = Redmine::Activity::Fetcher.new(User.current).events(nil, nil, :last_by_project => true).to_h
+      projects.each do |project|
+        project.instance_variable_set(:@last_activity_date, last_activities[project.id])
+      end
+    end
+  end
+
   private
 
   def update_inherited_members
@@ -1006,7 +1049,7 @@ class Project < ActiveRecord::Base
   def add_inherited_member_roles
     if inherit_members? && parent
       parent.memberships.each do |parent_member|
-        member = Member.find_or_new(self.id, parent_member.user_id)
+        member = Member.find_or_initialize_by(:project_id => self.id, :user_id => parent_member.user_id)
         parent_member.member_roles.each do |parent_member_role|
           member.member_roles <<
             MemberRole.new(:role => parent_member_role.role,
@@ -1098,7 +1141,7 @@ class Project < ActiveRecord::Base
 
     # Store status and reopen locked/closed versions
     version_statuses = versions.reject(&:open?).map {|version| [version, version.status]}
-    version_statuses.each do |version, status|
+    version_statuses.each do |version, _| # rubocop:disable Style/HashEachMethods
       version.update_attribute :status, 'open'
     end
 
@@ -1229,6 +1272,9 @@ class Project < ActiveRecord::Base
       new_query.user_id = query.user_id
       new_query.role_ids = query.role_ids if query.visibility == ::Query::VISIBILITY_ROLES
       self.queries << new_query
+      if query == project.default_issue_query
+        self.default_issue_query = new_query
+      end
     end
   end
 
@@ -1284,4 +1330,9 @@ class Project < ActiveRecord::Base
     end
     update_attribute :status, STATUS_ARCHIVED
   end
+
+  def fetch_last_activity_date
+    latest_activities = Redmine::Activity::Fetcher.new(User.current, :project => self).events(nil, nil, :last_by_project => true)
+    latest_activities.empty? ? nil : latest_activities.to_h[self.id]
+  end
 end
