@@ -1,7 +1,7 @@
 # frozen_string_literal: true
 
 # Redmine - project management software
-# Copyright (C) 2006-2023  Jean-Philippe Lang
+# Copyright (C) 2006-  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -34,6 +34,6 @@ class RedminePluginMigrationGenerator < Rails::Generators::NamedBase
     plugin_name = file_name.underscore
     plugin_path = File.join(Redmine::Plugin.directory, plugin_name)
     migration_template "migration.rb",
-                       "#{plugin_path}/db/migrate/#{@migration}.rb"
+                       "#{plugin_path}/db/migrate/#{@migration.underscore}.rb"
   end
 end
